#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé mai 2024

@auteur: david ALBERTO
(www.astrolabe-science.fr)
"""
import matplotlib.pyplot as plt
import numpy as np
from skyfield.api import Star, load
from skyfield.data import hipparcos
from skyfield.api import N,S,E,W, wgs84 #entrer coordonnées d'un lieu
from skyfield.api import Loader #ajuster le fichier de téléchargement d'éphémérides
import datetime as dt
from pytz import timezone
import locale #dates en français

plt.rcParams["font.family"] = "Roboto Condensed"

# Paramètres réglables par l'utilisateur :
latitude=20# degrés
taille_points=4
pasHeures=2
date_debut=dt.datetime(1500, 6, 15, 18, 0)# année, mois, jour, heure, min
date_fin=dt.datetime(1500, 6, 16, 8)# année, mois, jour, heure, min
prop_lignes = dict(lw=1)
# -----------------------------------------------
    
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')#dates en français

PSD=np.pi/2#pi sur 2

# Définition des étoiles de la Croix du Sud :
with load.open(hipparcos.URL) as f:
    df = hipparcos.load_dataframe(f)

# Rechercher le numéro d'une étoile dans le catalogue Hipparcos :
    # https://vizier.u-strasbg.fr/viz-bin/VizieR?-source=1239/hip_main
    # ou Stellarium
# Insérer ce numéro entre crochets :
acrucis = Star.from_dataframe(df.loc[60718])
gacrucis = Star.from_dataframe(df.loc[61084])
mimosa = Star.from_dataframe(df.loc[62434])
imai = Star.from_dataframe(df.loc[59747])

# liste_etoiles=[acrucis, gacrucis, mimosa, imai] # ordre choisi pour le tracé de l'astérisme
axe1 = [acrucis, gacrucis] # ordre choisi pour le tracé de l'astérisme
axe2 = [mimosa, imai] # ordre choisi pour le tracé de l'astérisme
liste_etoiles = [axe1[0], axe1[1], axe2[0], axe2[1]]
# -----------------------------------
# load = Loader('../skyfield-ephemerides')
# planets = load('de440.bsp')# 114MB autre fichier d'éphémérides (1550-2650)
# planets = load('de422.bsp')# 623MB autre fichier d'éphémérides (-3000-3000)
# planets = load('de422_excerpt.bsp')# 22MB autre fichier d'éphémérides (1450-1650)
planets = load('de422_excerpt.bsp')
#Paramètres du lieu d'observation :
earth = planets['earth']
lieu = earth + wgs84.latlon(latitude * S, 50 * W)
# -------------------------------

ts = load.timescale()

pasH=dt.timedelta(hours=pasHeures)

#Prise en compte du fuseau horaire :
zone = timezone('UTC')# fuseau horaire 
date_debut=zone.localize(date_debut)
date_fin=zone.localize(date_fin)
# --------------------------------------
# PARAMETRES DU GRAPHIQUE :
fig=plt.figure(figsize=(8,8),tight_layout=True, facecolor='lavender')
ax=plt.subplot(111,projection='polar', facecolor='white')
ax.set_theta_zero_location("N")
ax.set_theta_direction(-1) # inversion du sens de rotation (hémisphère sud)
ax.set_xticks([])
ax.set_title("Rotation de la constellation \n de la Croix du Sud", fontsize=14,y=0.96, loc='left')

# --------------------------------------
#  Tracé :
date = date_debut
compteur = 0
while date <= date_fin:
    couleur='C'+str(compteur)
    t=ts.from_datetime(date)
    liste_X, liste_Y = [], []
    for etoile in axe1:
        astrometric = lieu.at(t).observe(etoile)
        apparent = astrometric.apparent()
        # alt, az, distance = apparent.altaz()
        ha, dec, distance = apparent.hadec()
        liste_X.append(ha.radians)
        dist_polaire = PSD + dec.radians  # distance polaire de l'étoile  
        liste_Y.append(np.degrees(dist_polaire))
    ax.plot(liste_X, liste_Y, 'o-',
            c=couleur, **prop_lignes, mec='k', mew=0.4, ms=taille_points,
            label=date.strftime("%Hh%M"))
    liste_X, liste_Y = [], []
    for etoile in axe2:
        astrometric = lieu.at(t).observe(etoile)
        apparent = astrometric.apparent()
        # alt, az, distance = apparent.altaz()
        ha, dec, distance = apparent.hadec()
        liste_X.append(ha.radians)
        dist_polaire = PSD + dec.radians  # distance polaire de l'étoile  
        liste_Y.append(np.degrees(dist_polaire))
    ax.plot(liste_X, liste_Y, 'o-',
            c=couleur, **prop_lignes, mec='k', mew=0.4, ms=taille_points)
    # Étoile polaire :
    # x_pol.append(liste_X[0])
    # y_pol.append(liste_Y[0])
    date=date+pasH
    compteur+=1

# ax.set_yticks(np.arange(0,20,1), minor=True)
# ax.set_ylim(20, 40)
ax.set_rlabel_position(-90)#position des graduations des rayons
rtick_locs = np.arange(5,40,5)
rtick_labels = ['%i°'%r for r in rtick_locs]
ax.set_rgrids(rtick_locs, rtick_labels, fontsize=10, ha='center', va='center')

ax.grid(which='major', alpha=1, linewidth=0.7)
ax.grid(which='minor', alpha=0.6, linewidth=0.3)

#  pôle nord :
ax.scatter(0,0, marker='o', c='lightsalmon', ec='k', lw=0.4, label="pôle sud céleste")
ax.text(0,0,'P', ha='center', va='center', fontsize=5)

# Mise en forme de la légende (dates):
fig.subplots_adjust(bottom=0.24)#place pour la légende
legende = ax.legend(
    loc='lower center',
    title="À partir du "+date_debut.strftime('%d %b %Y'),
    ncol=5,
    bbox_to_anchor=(0.5,0.05),
    bbox_transform=fig.transFigure,
    markerscale=1,
    )

for texte in legende.get_texts():
    texte.set_fontsize(8)

legende.legendPatch.set_linewidth(1)
legende.legendPatch.set_edgecolor('purple')



NomFichier="RotationCroixSud"+date_debut.strftime("%Y")
# fig.savefig(NomFichier+"_V3.png",dpi=300)
fig.savefig(NomFichier+".pdf")
fig.savefig(NomFichier+".png",dpi=300)
